from __future__ import print_function
import os, logging, pprint

log = logging.getLogger(__name__)
if not os.environ.get("PYOPENGL_PLATFORM"):
    os.environ["PYOPENGL_PLATFORM"] = "egl"
if "DISPLAY" in os.environ:
    del os.environ["DISPLAY"]
import logging, contextlib
from functools import wraps

# from OpenGL.GL import *
from OpenGL.EGL import *
from OpenGL.EGL import debug
from OpenGL import arrays
from OpenGL.EGL import gbmdevice
from OpenGL.EGL.MESA import platform_gbm
from OpenGL.EGL.EXT import platform_device, platform_base, device_base

API_MAP = {
    EGL_OPENGL_BIT: EGL_OPENGL_API,
    EGL_OPENGL_ES2_BIT: EGL_OPENGL_ES_API,
    EGL_OPENGL_ES_BIT: EGL_OPENGL_ES_API,
}


def platformDisplay(device):
    """Get platform display from device specifier
    
    device -- EGLDeviceEXT, gbm card* path, or gbm card* ordinal (index)

    returns display, created_device (or None if passed in)
    raises RuntimeError if we can't create the display
    """
    created_device = display = None
    if isinstance(device, (str, int)):
        created_device = device = gbmdevice.open_device(device)
    if eglGetPlatformDisplay:
        display = eglGetPlatformDisplay(
            platform_device.EGL_PLATFORM_DEVICE_EXT
            if isinstance(device, EGLDeviceEXT)
            else platform_gbm.EGL_PLATFORM_GBM_MESA,
            device,
            ctypes.c_void_p(0),
        )
        if display == EGL_NO_DISPLAY:
            raise RuntimeError("Unable to create EGL display on %s" % (display))
    else:
        raise RuntimeError("eglGetPlatformDisplay has no implementation")
    if not created_device:
        try:
            name = get_device_name(device)
            if name is not None:
                log.debug("DRM Name: %s", name)
        except EGLError:
            log.debug("Unable to retrieve the DRM name")
    return display, created_device


def gbmPlatformSurface(display, config, platform_device, width, height):
    """Create a GBM platform surface for display with config on platform_device
    
    returns egl_surface, gbm_surface
    """
    visual = EGLint()
    eglGetConfigAttrib(display, config, EGL_NATIVE_VISUAL_ID, visual)
    platform_surface = gbmdevice.create_surface(
        platform_device,
        width,
        height,
        format=visual.value,
        flags=gbmdevice.GBM_BO_USE_RENDERING,
    )
    if not platform_surface:
        raise RuntimeError("Unable to allocate a gbm surface")
    surface = eglCreatePlatformWindowSurface(display, config, platform_surface, None)
    if surface == EGL_NO_SURFACE:
        log.error("Failed to create the EGL surface on the GBM surface")
        raise RuntimeError("Platform window surface creation failure")
    return surface, platform_surface


def choose_config(display, attributes):
    """utility to choose config for the display based on attributes"""
    num_configs = EGLint()
    configs = (EGLConfig * 1)()
    local_attributes = arrays.GLintArray.asArray(attributes)
    success = eglChooseConfig(display, local_attributes, configs, 1, num_configs)
    if not success:
        raise NoConfig("Unable to complete config filtering", attributes)
    if not num_configs:
        raise NoConfig(
            "No compatible configs found", attributes,
        )
    return configs[0]


@contextlib.contextmanager
def egl_context(
    width=256,
    height=256,
    api=EGL_OPENGL_BIT,
    attributes=(
        EGL_BLUE_SIZE,
        8,
        EGL_RED_SIZE,
        8,
        EGL_GREEN_SIZE,
        8,
        EGL_DEPTH_SIZE,
        24,
        EGL_COLOR_BUFFER_TYPE,
        EGL_RGB_BUFFER,
        # EGL_CONFIG_CAVEAT, EGL_NONE, # Don't allow slow/non-conformant
    ),
    pbuffer=False,
    device=None,
    output="output.ppm",
):
    """Setup a context for rendering"""
    major, minor = EGLint(), EGLint()
    created_device = platform_surface = surface = None
    if device is None:
        display = eglGetDisplay(EGL_DEFAULT_DISPLAY)
        if display == EGL_NO_DISPLAY:
            raise RuntimeError(EGL_NO_DISPLAY, "Could not create default display")
    else:
        display, created_device = platformDisplay(device)
    try:
        # print("Display: %s"%(display.address,))
        try:
            eglInitialize(display, major, minor)
        except EGLError as err:
            log.warning("eglInitilise failure on %s: %s", display, err.err)
            raise NoEGLSupport(display)
        log.debug(
            "Available configs:\n%s",
            debug.format_debug_configs(debug.debug_configs(display)),
        )

        # for config in configs[:num_configs.value]:
        #     log.debug("Config: %s",pprint.pformat(debug.debug_config(display,config)))
        local_attributes = list(attributes[:])
        if pbuffer:
            local_attributes.extend(
                [EGL_SURFACE_TYPE, EGL_PBUFFER_BIT,]
            )
        else:
            local_attributes.extend(
                [EGL_SURFACE_TYPE, EGL_WINDOW_BIT,]
            )
        local_attributes.extend(
            [EGL_CONFORMANT, api, EGL_NONE,]  # end of list
        )
        config = choose_config(display, local_attributes,)
        log.debug(
            "Selected config:\n%s",
            debug.format_debug_configs(debug.debug_configs(display, configs=[config])),
        )
        surface_attributes = [
            EGL_WIDTH,
            width,
            EGL_HEIGHT,
            height,
            EGL_NONE,
        ]
        if pbuffer:
            surface = eglCreatePbufferSurface(display, config, surface_attributes,)
        else:
            surface, platform_surface = gbmPlatformSurface(
                display, config, created_device, width, height
            )
        eglBindAPI(API_MAP[api])
        ctx = eglCreateContext(display, config, EGL_NO_CONTEXT, None)
        if ctx == EGL_NO_CONTEXT:
            raise RuntimeError("Unable to create context")
        eglMakeCurrent(display, surface, surface, ctx)
        yield display, ctx, surface
        if output:
            log.debug("Doing readpixels for writing buffer")
            from OpenGL import arrays

            content = arrays.GLubyteArray.zeros((width, height, 3))
            if api == EGL_OPENGL_BIT:
                from OpenGL.GL import glReadPixels, GL_UNSIGNED_BYTE, GL_RGB
            elif api == EGL_OPENGL_ES3_BIT:
                from OpenGL.GLES3 import glReadPixels, GL_UNSIGNED_BYTE, GL_RGB
            elif api == EGL_OPENGL_ES2_BIT:
                from OpenGL.GLES2 import glReadPixels, GL_UNSIGNED_BYTE, GL_RGB
            elif api == EGL_OPENGL_ES_BIT:
                from OpenGL.GLES1 import glReadPixels, GL_UNSIGNED_BYTE, GL_RGB
            content = glReadPixels(
                0, 0, width, height, GL_RGB, type=GL_UNSIGNED_BYTE, array=content
            )

            debug.write_ppm(content, output)
            # glFinish()
    finally:
        if display:
            eglMakeCurrent(display, None, None, None)
            if surface:
                eglDestroySurface(display, surface)
            eglTerminate(display)
        if platform_surface:
            gbmdevice.gbm.gbm_surface_destroy(platform_surface)
        if created_device:
            gbmdevice.close_device(created_device)


class NoEGLSupport(Exception):
    """Raised if we could not initialise an egl context"""


class NoConfig(Exception):
    """Raised if we did not find any configs"""


def debug_info(setup):
    from OpenGL.GL import (
        glClearColor,
        glClear,
        GL_COLOR_BUFFER_BIT,
        GL_DEPTH_BUFFER_BIT,
        glGetString,
        GL_VENDOR,
        GL_EXTENSIONS,
        GL_VERSION,
        glFinish,
    )

    display, ctx, surface = setup
    glClearColor(1.0, 1.0, 1.0, 1.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    log.info("Vendor: %s", glGetString(GL_VENDOR))
    log.info("Vendor: %s", glGetString(GL_VERSION))
    log.info("Extensions: %s", glGetString(GL_EXTENSIONS))
    glFinish()

def get_device_name(device):
    """Try to get the display's DRM device name

    This is almost certainly not going to work on
    anything other than Linux
    """
    from OpenGL.EGL.EXT.device_query import (
        eglQueryDeviceStringEXT,
    )
    from OpenGL.EGL.EXT.device_drm import (
        EGL_DRM_DEVICE_FILE_EXT,
    )
    if eglQueryDeviceStringEXT:
        name = eglQueryDeviceStringEXT(
            device,
            EGL_DRM_DEVICE_FILE_EXT
        )
        return name.decode('ascii',errors='ignore')
    return None


def main():
    # NOTE: having two different implementations here is
    # likely somewhat broken due to the
    # OpenGL functions having retrieved their
    # function pointers during the first pass and then
    # trying to run them against the second
    for device in sorted(device_base.egl_get_devices(), key=lambda x: x.address):
        log.info("Starting tests with: %s", device)
        try:
            with egl_context(device=device, pbuffer=True) as setup:
                debug_info(setup)
        except (NoEGLSupport, NoConfig) as err:
            log.info("Cannot configure: %s", err)
        except (EGLError, RuntimeError):
            log.exception("Failed during: %s", device)
    for device in gbmdevice.enumerate_devices():
        log.info("Starting tests with: %s", device)
        try:
            with egl_context(device=device, pbuffer=False) as setup:
                debug_info(setup)
        except (NoEGLSupport, NoConfig) as err:
            log.info("Cannot configure: %s", err)
        except (EGLError, RuntimeError):
            log.exception("Failed during: %s", device)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    main()



#! /usr/bin/env python
"""Implements the functionality in MESA's EGL + OpenGL demo1:

    http://cgit.freedesktop.org/mesa/demos/tree/src/egl/opengl/demo1.c

Basically just to check if the EGL code works...

NOTE: this test program is both MIT (MESA) and GPLv2 licensed due to use of 
sample code from python-Xlib.
"""
import pygame.display
import pygame
import os
import logging
import numpy

if not os.environ.get("PYOPENGL_PLATFORM"):
    os.environ["PYOPENGL_PLATFORM"] = "egl"

import OpenGL, ctypes

OpenGL.setPlatform("egl")

if os.environ.get("TEST_NO_ACCELERATE"):
    OpenGL.USE_ACCELERATE = False
from OpenGL._bytes import as_str
from OpenGL.EGL import *
from OpenGL.error import GLError

log = logging.getLogger(__name__)
import sys
from OpenGL.arrays.vbo import VBO


def describe_config(display, config):
    """Describe the given configuration"""
    parameters = (
        EGL_CONFIG_ID,
        EGL_CONFIG_CAVEAT,
        EGL_CONFORMANT,
        EGL_LEVEL,
        EGL_NATIVE_RENDERABLE,
        EGL_NATIVE_VISUAL_ID,
        EGL_NATIVE_VISUAL_TYPE,
        EGL_SURFACE_TYPE,
        EGL_ALPHA_SIZE,
        EGL_ALPHA_MASK_SIZE,
        EGL_BIND_TO_TEXTURE_RGB,
        EGL_BIND_TO_TEXTURE_RGBA,
        EGL_BLUE_SIZE,
        EGL_BUFFER_SIZE,
        EGL_COLOR_BUFFER_TYPE,
        EGL_DEPTH_SIZE,
        EGL_GREEN_SIZE,
        EGL_LUMINANCE_SIZE,
        EGL_MAX_PBUFFER_WIDTH,
        EGL_MAX_PBUFFER_HEIGHT,
        EGL_MAX_PBUFFER_PIXELS,
        EGL_MAX_SWAP_INTERVAL,
        EGL_MIN_SWAP_INTERVAL,
        EGL_RED_SIZE,
        EGL_RENDERABLE_TYPE,
        EGL_SAMPLE_BUFFERS,
        EGL_SAMPLES,
        EGL_STENCIL_SIZE,
        EGL_TRANSPARENT_TYPE,
        EGL_TRANSPARENT_RED_VALUE,
        EGL_TRANSPARENT_GREEN_VALUE,
        EGL_TRANSPARENT_BLUE_VALUE,
    )
    description = []
    for param in parameters:
        value = ctypes.c_long()
        eglGetConfigAttrib(display, config, param, value)
        description.append(
            "%s = %s"
            % (
                param,
                value.value,
            )
        )
    return "\n".join(description)


def mainloop(displayfunc):
    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return
        if not displayfunc():
            return False
        return True


def main(displayfunc, api):
    major, minor = ctypes.c_long(), ctypes.c_long()
    display = eglGetDisplay(EGL_DEFAULT_DISPLAY)
    log.info("Display return value: %s", display)
    log.info("Display address: %s", display.address)
    # display = display.as_voidp
    # print 'wrapped', display
    if not eglInitialize(display, major, minor):
        log.error("Unable to initialize")
        return
    log.info("EGL version %s.%s", major.value, minor.value)

    num_configs = ctypes.c_long()
    eglGetConfigs(display, None, 0, num_configs)
    log.info("%s configs", num_configs.value)

    configs = (EGLConfig * num_configs.value)()
    eglGetConfigs(display, configs, num_configs.value, num_configs)

    bit = EGL_OPENGL_API
    if api == 'gles':
        bit = EGL_OPENGL_ES_BIT
    elif api == 'gles2':
        bit = EGL_OPENGL_ES2_BIT
    attributes = [
        EGL_NATIVE_RENDERABLE,
        EGL_TRUE,
        EGL_CONFORMANT,
        bit,
        EGL_COLOR_BUFFER_TYPE,
        EGL_RGB_BUFFER,
        EGL_NONE,
    ]
    attributes = (EGLint * len(attributes))(*attributes)
    eglChooseConfig(display, attributes, configs, len(configs), num_configs)

    for number, config_id in enumerate(configs):
        # print config_id
        log.info("Config #%d\n%s", number, describe_config(display, config_id))
        break

    log.info("Attempting to bind and create contexts/apis for %s", api)
    try:
        eglBindAPI(0x3333)  # junk value
    except GLError:
        pass
    else:
        assert False, "Should have generated error on bind to non-existent api"
    eglBindAPI(api)

    # now need to get a raw X window handle...
    pygame.init()

    pygame.display.set_mode((500, 500), flags=pygame.NOFRAME | pygame.SHOWN)
    # pygame.display.init()
    window = pygame.display.get_wm_info()["window"]

    # print("Clearing current context")
    # eglMakeCurrent(display, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT)

    surface = eglCreateWindowSurface(display, configs[0], window, None)
    if surface == EGL_NO_SURFACE:
        raise RuntimeError("No surface could be created")

    ctx = eglCreateContext(display, configs[0], EGL_NO_CONTEXT, None)
    if ctx == EGL_NO_CONTEXT:
        log.error("Unable to create the regular context")
        return
    else:
        log.info("Created regular context")

        def _displayfunc():
            try:
                displayfunc(display, surface, ctx)
            except Exception:
                log.exception("Failure during display function")
                return False
            else:
                return True

        if not mainloop(_displayfunc):
            raise RuntimeError("Display func crashed")

    pbufAttribs = (EGLint * 5)(*[EGL_WIDTH, 500, EGL_HEIGHT, 500, EGL_NONE])
    pbuffer = eglCreatePbufferSurface(display, configs[0], pbufAttribs)
    if pbuffer == EGL_NO_SURFACE:
        log.error("Unable to create pbuffer surface")
    else:
        log.info("created pbuffer surface")
    log.info(
        "Available EGL extensions:\n  %s",
        "\n  ".join([as_str(ext) for ext in EGLQuerier.getExtensions().split()]),
    )
    print('OK')


def displayfunc_gl(display, surface, ctx):
    from OpenGL import GL

    eglMakeCurrent(display, surface, surface, ctx)
    GL.glClearColor(1, 0, 0, 0)
    GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
    eglSwapBuffers(display, surface)


displayfunc_gl.api = EGL_OPENGL_API


def displayfunc_gles1(display, surface, ctx):
    from OpenGL import GLES1 as GL

    eglMakeCurrent(display, surface, surface, ctx)
    GL.glClearColor(1, 0, 0, 0)
    GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
    vertices = numpy.array(((1, 0, 0), (-1, 0, 0), (0, 1, 0)), "f")
    GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
    GL.glVertexPointer(3, GL.GL_FLOAT, 0, vertices)
    GL.glDrawArrays(GL.GL_TRIANGLES, 0, 3)
    eglSwapBuffers(display, surface)


displayfunc_gles1.api = EGL_OPENGL_ES_API


shader = None


def displayfunc_gles2(display, surface, ctx):
    from OpenGL import GLES2 as GL
    from OpenGL.GLES2 import shaders

    eglMakeCurrent(display, surface, surface, ctx)
    GL.glClearColor(1, 0, 0, 0)
    GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

    global shader, vbo, position_location, stride
    if shader is None:
        shader = shaders.compileProgram(
            shaders.compileShader(
                """#version 130
    attribute vec3 position;
    void main() {
        gl_Position = vec4( position, 0 );
    }""",
                type=GL_VERTEX_SHADER,
            ),
            shaders.compileShader(
                """#version 130
    void main() {
        gl_FragColor = vec4( 1,0,0,0 );
    }""",
                type=GL_FRAGMENT_SHADER,
            ),
        )
        vbo = VBO(
            array(
                [
                    (0, 1, 0),
                    (1, -1, 0),
                    (-1, -1, 0),
                ],
                dtype="f",
            )
        )
        position_location = GL.glGetAttribLocation(self.shader, "position")
        stride = 3 * 4
    with vbo:
        with shader:
            GL.glEnableVertexAttribArray(position_location)
            stride = 3 * 4
            GL.glVertexAttribPointer(
                position_location, 3, GL_FLOAT, False, stride, self.vbo
            )
            GL.glDrawArrays(GL.GL_TRIANGLES, 0, 3)
    eglSwapBuffers(display, surface)


displayfunc_gles2.api = EGL_OPENGL_ES_API


def displayfunc_gles3(display, surface, ctx):
    from OpenGL import GLES3 as GL

    eglMakeCurrent(display, surface, surface, ctx)
    GL.glClearColor(1, 0, 0, 0)
    GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
    eglSwapBuffers(display, surface)


displayfunc_gles3.api = EGL_OPENGL_ES_API


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    if sys.argv[1:]:
        name = sys.argv[1]
    else:
        name = "gl"
    if name == "gles":
        name = "gles1"
    function = globals().get("displayfunc_%s" % (name,), displayfunc_gl)
    log.info("Using function: %s", function)
    main(function, function.api)



"""Transliteration of https://open.gl/feedback into Python"""

from __future__ import print_function
import pygamegltest
from OpenGL.GL import *
from OpenGL.GL import shaders

vertex_shader = """#version 150 core
    in float inValue;
    out float outValue;

    void main() {
        outValue = sqrt(inValue);
    }
"""


@pygamegltest.pygametest(name="Geometry Shader Test")
def main():
    program = shaders.compileProgram(
        shaders.compileShader([vertex_shader], GL_VERTEX_SHADER),
    )

    buff = ctypes.c_char_p(b"outValue")
    c_text = ctypes.cast(ctypes.pointer(buff), ctypes.POINTER(ctypes.POINTER(GLchar)))
    # modifies the state in the linking of the program
    glTransformFeedbackVaryings(program, 1, c_text, GL_INTERLEAVED_ATTRIBS)

    # so have to re-link
    glLinkProgram(program)
    glUseProgram(program)

    vao = glGenVertexArrays(1)
    glBindVertexArray(vao)

    data = (GLfloat * 5)(1.0, 2.0, 3.0, 4.0, 5.0)

    vbo = glGenBuffers(1)
    glBindBuffer(GL_ARRAY_BUFFER, vbo)
    glBufferData(GL_ARRAY_BUFFER, ctypes.sizeof(data), data, GL_STATIC_DRAW)

    inputAttrib = glGetAttribLocation(program, "inValue")
    glEnableVertexAttribArray(inputAttrib)
    # Note the need to cast 0 to a GLvoidp here!
    glVertexAttribPointer(inputAttrib, 1, GL_FLOAT, GL_FALSE, 0, GLvoidp(0))

    tbo = glGenBuffers(1)
    glBindBuffer(GL_ARRAY_BUFFER, tbo)
    glBufferData(GL_ARRAY_BUFFER, ctypes.sizeof(data), None, GL_STATIC_READ)

    glEnable(GL_RASTERIZER_DISCARD)

    glBindBufferBase(GL_TRANSFORM_FEEDBACK_BUFFER, 0, tbo)

    glBeginTransformFeedback(GL_POINTS)
    glDrawArrays(GL_POINTS, 0, len(data))
    glEndTransformFeedback()

    glDisable(GL_RASTERIZER_DISCARD)

    glFlush()

    feedback = (GLfloat * len(data))()
    glGetBufferSubData(
        GL_TRANSFORM_FEEDBACK_BUFFER, 0, ctypes.sizeof(feedback), feedback
    )

    for value in feedback:
        print(value)
    print('OK')


if __name__ == "__main__":
    main()



#! /usr/bin/env python
import xlibegltest as egltest
import numpy
from OpenGL import GLES1 as GL


@egltest.egltest(api='gles1')
def test_es1():
    GL.glClearColor(1, 0, 0, 0)
    GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
    vertices = numpy.array(((1, 0, 0), (-1, 0, 0), (0, 1, 0)), 'f')
    GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
    GL.glVertexPointer(3, GL.GL_FLOAT, 0, vertices)
    GL.glDrawArrays(GL.GL_TRIANGLES, 0, 3)
    print('OK')


if __name__ == "__main__":
    test_es1()



#! /usr/bin/env python
"""Script to check our glgetsizes against regal's..."""
from __future__ import print_function
import os, logging, subprocess
import codegenerator, ctypetopytype, xmlreg
log = logging.getLogger( __name__ )
HERE = os.path.dirname( __file__ )
REGAL = os.path.join( HERE, 'regal' )
def update_regal():
    if not os.path.exists( REGAL ):
        command= 'git clone git@github.com:p3/regal.git'
    else:
        command= 'cd %s && git pull'%(REGAL,)
    log.info( 'Running: %s', command )
    subprocess.check_call( command, shell=True )
    return os.path.join( REGAL,'src','apitrace','wrappers','regaltrace.cpp' )

def load_chromium():
    content = open( update_regal() ).read()
    content = content[content.index('_gl_param_size('):]
    content = content.splitlines()[1:]
    mapping = {}
    for line in content:
        line = line.strip()
        if line.startswith( 'default' ):
            break
        if line.startswith( 'case' ):
            line = line[5:]
            var,rvalue = line.split(':')
            var = var.strip()
            rvalue = rvalue.strip().strip(';')
            if rvalue.startswith('return'):
                rvalue = rvalue[6:].strip()
            try:
                rvalue = int( rvalue )
            except ValueError:
                log.warning( 'Complex return for: %s', rvalue )
                continue 
            else:
                if rvalue == 16:
                    rvalue = (4,4)
                else:
                    rvalue = (rvalue,)
            mapping[var] = rvalue
    return mapping
def main():
    chromium = load_chromium()
    registry = xmlreg.parse( os.path.join( HERE,'khronosapi','gl.xml') )
    generator = codegenerator.Generator(
        registry,
        ctypetopytype.ctype_to_pytype
    )
    for key,value in generator.glGetSizes.items():
        if not value:
            if key in chromium:
                print('Updating %s should be %s'%( key, chromium[key] ))
                generator.glGetSizes[key] = [str(chromium[key]).replace(' ','')]
        try:
            chromium.pop(key)
        except KeyError:
            pass 
    for key,value in chromium.items():
        print('New constant from chromium:', (key,value))
        generator.glGetSizes[key] = [str(chromium[key]).replace(' ','')]
    generator.saveGLGetSizes()
    
if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO )
    main()



#! /usr/bin/env python
"""Check for namespace polution..."""
from __future__ import print_function

current = set(dir())
from OpenGL.GL import *
new = set(dir())
unexpected = sorted([
    x 
    for x in (new-current-set(['current','OpenGL'])) 
    if not x.lower().startswith( 'gl' )
])

for name in unexpected:
    print(name)



"""Refactored version of the opengl generator using ctypeslib
"""
try:
    from ctypeslib.codegen import codegenerator
    from ctypeslib import xml2py
except ImportError as err:
    try:
        from ctypes_codegen import codegenerator, xml2py
    except ImportError as err:
        from ctypes.wrap import codegenerator, xml2py
try:
    from cStringIO import StringIO
except ImportError as err:
    from StringIO import StringIO

import sys, logging

log = logging.getLogger('openglgenerator')
import ctypes
from OpenGL.platform import GL, GLU, GLUT, GLE
from OpenGL import constant


def indent(code, indentation='\t'):
    """Indent given code by given indentation"""
    lines = code.splitlines()
    return "\n".join(['%s%s' % (indentation, line) for line in lines])


class OpenGLGenerator(codegenerator.Generator):
    """Subclass of code generator providing PyOpenGL integration"""

    _super = codegenerator.Generator
    MODULE_HEADER = """from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GL_types.GLvoid
"""

    @classmethod
    def defaultEmitters(cls):
        """Produce the set of default emitter classes"""
        return [
            OpenGLFunction(),
            OpenGLConstant(),
        ] + cls._super.defaultEmitters()

    @classmethod
    def importAble(cls, name, value):
        """Determine whether this name/object should be imported from known symbols"""
        return (
            isinstance(value, type)
            or isinstance(value, constant.Constant)
            or value.__class__.__name__.endswith(
                'CFunctionType'
            )  # this should be available *somewhere*!
        )

    def filter_items(self, items, expressions=None, symbols=None, types=None):
        """Filter out PFN functions"""
        items = [
            i
            for i in items
            # skip the pointer-to-function meta-types...
            if not getattr(i, 'name', '').startswith('PFN')
        ]
        return self._super.filter_items(
            self, items, expressions=expressions, symbols=symbols, types=types
        )

    def get_sharedlib(self, dllname, cc):
        """Override so that all references to shared libraries go through "platform" module"""
        if dllname in ('libGL', 'GL', 'libGL.so.1'):
            return 'platform.PLATFORM.GL'
        elif dllname in ('libGLU', 'GLU', 'libGLU.so.1'):
            return 'platform.PLATFORM.GLU'
        elif dllname in ('libglut', 'glut', 'libglut.so.3'):
            return 'platform.PLATFORM.GLUT'
        elif dllname in ('libgle', 'gle', 'libgle.so.3'):
            return 'platform.PLATFORM.GLE'
        else:
            raise NotImplementedError("""Haven't done %s yet!""" % (dllname))

    def cmpitems(self, a, b):
        """Dumb sorting helper to order by name instead of position"""
        try:
            return cmp(
                (a.name, getattr(a, "location", -1), a.__class__),
                (b.name, getattr(b, "location", 1), b.__class__),
            )
        except (AttributeError, TypeError, ValueError) as err:
            return cmp(a, b)


class OpenGLFunction(codegenerator.Function):
    """Replaces the ctypes default code generator for functions"""

    TEMPLATE = """%(location)s%(name)s = platform.createBaseFunction( 
    %(name)r, dll=%(libname)s, resultType=%(returnType)s, 
    argTypes=[%(argTypes)s],
    doc=%(documentation)r, 
    argNames=%(argNames)r,
)
"""

    def emit(self, generator, func):
        """Produce a function via a call to platform-provided function"""
        result = []
        libname = self.libName(generator, func)
        if libname:
            self.increment()
            result.append(self.generateHeader(generator, func))
            args = self.getArgs(generator, func)
            argTypes = ",".join(args)
            argNames = self.getArgNames(generator, func)
            location = self.locationComment(generator, func)
            name = func.name
            returnType = generator.type_name(func.returns)

            documentation = self.documentFunction(generator, func)

            generator.names.add(func.name)
            result.append(self.TEMPLATE % locals())
            return result
        elif not func.name.startswith('__builtin_'):
            log.warning("""Could not find DLL name for function: %r""", func.name)
            return ''

    def arrayTypeName(self, generator, argType):
        """Retrieve the array type name for argType or None"""
        if generator.type_name(argType).startswith('POINTER'):
            # side effect should be to make the type available,
            # but doesn't work with GLvoid
            typeName = generator.type_name(argType.typ)
            if typeName in self.CTYPE_TO_ARRAY_TYPE:
                return 'arrays.%s' % (self.CTYPE_TO_ARRAY_TYPE[typeName])
            elif typeName == 'GLvoid':
                # normal to not have pointers to it...
                log.info('GLvoid pointer %r, using POINTER(%s)', typeName, typeName)
            else:
                log.warning('No mapping for %r, using POINTER(%s)', typeName, typeName)
        return None

    def getArgs(self, generator, func):
        """Retrieve arg type-names for all arguments in function typedef"""
        return [
            self.arrayTypeName(generator, a) or generator.type_name(a)
            for a in func.iterArgTypes()
        ]

    def documentFunction(self, generator, func):
        """Customisation point for documenting a given function"""
        args = self.getArgs(generator, func)
        argnames = self.getArgNames(generator, func)
        return str(
            "%s( %s ) -> %s"
            % (
                func.name,
                ", ".join(
                    ['%s(%s)' % (name, typ) for (name, typ) in zip(args, argnames)]
                ),
                generator.type_name(func.returns),
            )
        )

    SUFFIX_TO_ARRAY_DATATYPE = [
        ('ub', 'GLGL_1_0.GL_UNSIGNED_BYTE'),
        ('us', 'GLGL_1_0.GL_UNSIGNED_SHORT'),
        ('ui', 'GLGL_1_0.GL_UNSIGNED_INT'),
        ('f', 'GLGL_1_0.GL_FLOAT'),
        ('d', 'GLGL_1_0.GL_DOUBLE'),
        ('i', 'GLGL_1_0.GL_INT'),
        ('s', 'GLGL_1_0.GL_SHORT'),
        ('b', 'GLGL_1_0.GL_BYTE'),
    ]
    CTYPE_TO_ARRAY_TYPE = {
        'GLfloat': 'GLfloatArray',
        'float': 'GLfloatArray',
        'GLclampf': 'GLclampfArray',
        'GLdouble': 'GLdoubleArray',
        'double': 'GLdoubleArray',
        'int': 'GLintArray',
        'GLint': 'GLintArray',
        'GLuint': 'GLuintArray',
        'unsigned int': 'GLuintArray',
        'unsigned char': 'GLbyteArray',
        'uint': 'GLuintArray',
        'GLshort': 'GLshortArray',
        'GLushort': 'GLushortArray',
        'short unsigned int': 'GLushortArray',
        'GLubyte': 'GLubyteArray',
        'GLbyte': 'GLbyteArray',
        'char': 'GLbyteArray',
        'gleDouble': 'GLdoubleArray',
        # following should all have special sub-classes that enforce dimensions
        'gleDouble * 4': 'GLdoubleArray',
        'gleDouble * 3': 'GLdoubleArray',
        'gleDouble * 2': 'GLdoubleArray',
        'c_float * 3': 'GLfloatArray',
        'gleDouble * 3 * 2': 'GLdoubleArray',
    }


class OpenGLConstant(codegenerator.Variable):
    """Override to produce OpenGL.constant.Constant instances"""

    TEMPLATE = """%(name)s = Constant( %(name)r, %(value)r)"""

    def emit(self, generator, typedef):
        """Filter out constants that don't have all-uppercase names"""
        if typedef.name.upper() != typedef.name:
            return ""
        return super(OpenGLConstant, self).emit(generator, typedef)


class OpenGLDecorator(OpenGLFunction):
    """Produces decorated versions of the functions in a separate module

    This is passed in as an emitter for a separate pass, so that only the
    annotations get into the separate module.
    """

    def isPointer(self, generator, arg):
        """Is given arg-type a pointer?"""
        return generator.type_name(arg).startswith('POINTER')

    def hasPointer(self, generator, args):
        """Given set of arg-types, is one a pointer?"""
        return [arg for arg in args if self.isPointer(generator, arg)]

    def emit(self, generator, func):
        """Emit code to create a copy of the function with pointer-size annotations"""
        name = func.name
        size = None
        typ = None
        if not self.hasPointer(generator, func.iterArgTypes()):
            return None
        libname = self.libName(generator, func)
        if not libname:
            return None
        base = name
        if name.endswith('ARB'):
            base = base[:-3]
        if base.endswith('v'):
            base = base[:-1]
            found = 0
            for suffix, typ in self.SUFFIX_TO_ARRAY_DATATYPE:
                if base.endswith(suffix):
                    found = 1
                    base = base[: -len(suffix)]
                    try:
                        size = int(base[-1])
                    except ValueError as err:
                        size = None
                    break
        elif base[:-1].endswith('Matrix'):
            # glLoadMatrix, glMultMatrix
            for suffix, typ in self.SUFFIX_TO_ARRAY_DATATYPE:
                if name.endswith(suffix):
                    size = 16
                    break
        result = ''
        for index, (arg, argName) in enumerate(
            zip(func.iterArgTypes(), func.iterArgNames())
        ):
            type = self.arrayTypeName(generator, arg)
            argName = str(argName)
            if type:
                generator.names.add(func.name)
            if result:
                previous = indent(result, '\t')
            else:
                previous = '\traw.%(name)s' % locals()
            if type and size is None:
                # should only print this if it's a normal array type...
                result = (
                    """arrays.setInputArraySizeType(
%(previous)s,
    None, # XXX Could not determine size of argument %(argName)s for %(name)s %(type)s
    %(type)s, 
    %(argName)r,
)
"""
                    % locals()
                )
            elif type:
                result = (
                    """arrays.setInputArraySizeType(
%(previous)s,
    %(size)s,
    %(type)s,
    %(argName)r,
)
"""
                    % locals()
                )
        if result:
            return '%(name)s = %(result)s' % locals()
        return None


if __name__ == "__main__":
    import sys, logging

    logging.basicConfig()
    codegenerator.Generator = OpenGLGenerator
    sys.exit(xml2py.main())



from __future__ import print_function
import sys
from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.extensions import alternate
from OpenGL.GL.ARB.draw_instanced import *
from OpenGL.GL.EXT.draw_instanced import *


def detect():
    glutInit([])
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH)
    glutInitWindowSize(200, 200)

    glutCreateWindow("Detection")

    functions = (
        glDrawArraysInstanced,
        glDrawArraysInstancedARB,
        glDrawArraysInstancedEXT,
        # glDrawArraysInstancedNV, is GLES only
    )

    for function in functions:
        print(function.__name__, bool(function))

    any = alternate(*functions)

    print(any, bool(any))
    sys.stdout.write('OK\n')
    sys.stdout.flush()
    try:
        if fgDeinitialize:
            fgDeinitialize(False)
    except NameError:
        pass  # Older PyOpenGL, you may see a seg-fault here...


if __name__ == "__main__":
    detect()



import pygame as pg
import OpenGL

OpenGL.setPlatform("wayland")
import OpenGL.GL
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GL.ARB.framebuffer_object import glInitFramebufferObjectARB


def main():
    pg.init()
    pg.display.set_mode((50, 50), DOUBLEBUF | OPENGL)
    pg.display.init()
    OpenGL.GL.glGetString(OpenGL.GL.GL_VERSION)
    if not glInitFramebufferObjectARB:
        print("SKIP")
        return
    print(glInitFramebufferObjectARB())
    print(OpenGL.__version__)
    print('OK')


if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""Generate python PyOpenGL api from xml registry documents (using xmlreg)"""
from __future__ import print_function
import os, logging, subprocess, glob
import xmlreg, codegenerator
import ctypetopytype

log = logging.getLogger('xml-generate')

KHRONOS_URL = 'https://github.com/KhronosGroup/OpenGL-Registry.git'
KHRONOS_API = os.path.join(os.path.dirname(__file__), 'khronosapi')


def get_khronos(khronosapi):
    if not os.path.exists(khronosapi):
        log.info("Cloning the Khronos API into %s", khronosapi)
        subprocess.check_call(['git', 'clone', KHRONOS_URL, khronosapi])
    else:
        log.info("Pulling new commits from Khronos API")
        subprocess.check_call(['git', 'pull', '--ff-only'], cwd=khronosapi)


def main(khronosapi=None):
    khronosapi = khronosapi or KHRONOS_API
    get_khronos(khronosapi)

    files = sorted(glob.glob(os.path.join(khronosapi, 'xml', '*.xml')))
    for file in files:
        generate_for_file(file)


def generate_for_file(filename):
    log.info('Starting file: %s', filename)
    registry = xmlreg.parse(filename)
    generator = codegenerator.Generator(registry, ctypetopytype.ctype_to_pytype)
    for name, feature in registry.feature_set.items():
        print(feature.name, feature.api)
        generator.module(feature)
    for name, extension in registry.extension_set.items():
        print(extension.name, extension.apis)
        generator.module(extension)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()



from __future__ import print_function
import ctypes, sys

try:
    from ctypes.util import find_library
except ImportError as err:
    from ctypes.util import findLib

    def find_library(string):
        return findLib(string)[0]


GL = OpenGL = ctypes.CDLL(find_library('GL'), mode=ctypes.RTLD_GLOBAL)
GLU = ctypes.CDLL(find_library('GLU'), mode=ctypes.RTLD_GLOBAL)
# glut shouldn't need to be global IIUC
GLUT = ctypes.CDLL(find_library('glut'), mode=ctypes.RTLD_GLOBAL)
print('GL', GL)
print('GLU', GLU)
print('GLUT', GLUT)
print(GLUT.glutSolidTeapot)
sys.stdout.write('OK\n')
sys.stdout.flush()



'''OpenGL extension EXT.color_buffer_half_float

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.color_buffer_half_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows 16-bit floating point formats as defined in
	OES_texture_half_float to be rendered to via framebuffer objects.
	
	When using floating-point formats, certain color clamps are disabled.
	
	This extension also updates the framebuffer object API to allow querying
	attachment component types.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/color_buffer_half_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.color_buffer_half_float import *
from OpenGL.raw.GLES2.EXT.color_buffer_half_float import _EXTENSION_NAME

def glInitColorBufferHalfFloatEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""This checking script from https://github.com/mcfletch/pyopengl/issues/6"""

import OpenGL
import OpenGL.platform.egl

OpenGL.platform.PLATFORM = p = OpenGL.platform.egl.EGLPlatform()
from OpenGL import EGL
from OpenGL.EGL.VERSION import EGL_1_5
from OpenGL.EGL.EXT import platform_base
from OpenGL.EGL.MESA import platform_gbm
import ctypes, glob


def main():
    cards = sorted(glob.glob("/dev/dri/renderD*"))
    if not cards:
        print('SKIP')
        raise RuntimeError("Need a /dev/dri/renderD* device to do rendering")
    if len(cards) > 1:
        print("Note, using first card: %s" % (cards[0]))
    with open(cards[0], "w") as f:
        gbm = ctypes.CDLL("libgbm.so.1")  # On Ubuntu, package libgbm1
        dev = gbm.gbm_create_device(f.fileno())
        dpy = platform_base.eglGetPlatformDisplayEXT(
            platform_gbm.EGL_PLATFORM_GBM_MESA, ctypes.c_void_p(dev), ctypes.c_void_p(0)
        )
        print(dpy)
        if EGL_1_5.eglGetPlatformDisplay:
            dpy = EGL_1_5.eglGetPlatformDisplay(
                platform_gbm.EGL_PLATFORM_GBM_MESA,
                ctypes.c_void_p(dev),
                ctypes.c_void_p(0),
            )
            print(dpy)
        else:
            print("No EGL_1_5 implementation")
        print('OK')


if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""Test that GLUT.glutInit accepts 0 arguments"""
from __future__ import print_function
from OpenGL import GLUT

if __name__ == "__main__":
    try:
        GLUT.glutInit()
    except Exception as err:
        raise
    else:
        print('accepted 0 arguments, as desired')
        print('OK')
        try:
            if fgDeinitialize:
                fgDeinitialize(False)
        except NameError as err:
            pass  # Older PyOpenGL, you may see a seg-fault here...



#! /usr/bin/env python
"""Overall test runner to test matrix of system options..."""
import os,sys,subprocess,logging, glob, shutil
log = logging.getLogger( 'overallrunner' )
HERE = os.path.abspath(os.path.dirname( __file__ ))
TEST_ENVS = os.path.join( HERE, '.test-envs' )
WHEEL_DIR = os.path.join( HERE, '.wheels' )

PYTHONS = [
    # order is so that most-important platforms are checked first
    'python3.10',
    'python3.9',
    'python3.8',
    'python3.6',
    'python2.7',
    # python2.6 support is less important than the above at this point,
    # and doing a --user install clobbers 2.7's version of the packages
    # should use a virtualenv for all of them, really...
    #'python2.6', 
]
PYGAME_SOURCE = os.path.join( HERE, '.pygame' )

FLAGS = [
    'USE_ACCELERATE',
    'ERROR_ON_COPY',
    'ARRAY_SIZE_CHECKING',
    'ERROR_CHECKING',
    'STORE_POINTERS',
    'CONTEXT_CHECKING',
    'ALLOW_NUMPY_SCALARS',
    'UNSIGNED_BYTE_IMAGES_AS_STRING',
]

def short_name( python ):
    if python.startswith( 'python' ):
        return 'cp' + python[6:].replace('.','')
    raise ValueError( "Don't know wheel format for %s"%(python,))
def have_wheel( package,python ):
    """Do we have this wheel built for the given package and python?"""
    current = os.listdir( WHEEL_DIR )
    short = short_name( python )
    for item in current:
        split = item.split('-')
        if package in item and short in item:
            return True 
    return False

def get_pygame():
    if os.path.exists( PYGAME_SOURCE ):
        subprocess.check_call( 'cd .pygame && hg pull && hg update', shell=True )
    else:
        subprocess.check_call( 'hg clone https://bitbucket.org/pygame/pygame .pygame', shell=True )
    return PYGAME_SOURCE
def pip_command( python, command, **named ):
    if python == 'python2.6':
        prefix = ['pip2.6']
    else:
        prefix = [
            python, '-m', 'pip'
        ]
    command = prefix + command 
    return subprocess.check_call( command, **named )
    
def build_pygame(pythons=PYTHONS):
    get_pygame()
    cwd = os.path.join( HERE, '.pygame' )
    
    for python in [p for p in pythons if have_python(p)]:
        for path in [
            os.path.join(cwd, 'Setup' ),
            os.path.join(cwd, 'build' ),
        ]:
            if os.path.isdir( path ):
                shutil.rmtree( path )
            elif os.path.exists( path ):
                os.remove( path )
        subprocess.check_call( [python,'config.py'], cwd=cwd )
        pip_command( python, [
            'install','--user','--upgrade','pip',
        ], cwd=cwd )
        pip_command( python, [
            'install', '--upgrade','--user','wheel',
        ], cwd=cwd )
        
        if not have_wheel( 'pygame', python ):
            pip_command( python, [
                'wheel','--wheel-dir',WHEEL_DIR, '.pygame',
            ], cwd=HERE )
        if not have_wheel( 'numpy', python ):
            pip_command( python, [
                'wheel','--wheel-dir',WHEEL_DIR, '.numpy',
            ], cwd=HERE )

def have_python( python ):
    try:
        subprocess.check_call( ['which',python])
    except subprocess.CalledProcessError as err:
        log.error( "Unable to find %s", python )
        return None
    else:
        return python 

def has_module( python, module ):
    try:
        subprocess.check_call( [python, '-c', 'import %s'%(module)])
        return True 
    except subprocess.CalledProcessError as err:
        return False 
def has_pygame( python ):
    return has_module( python, 'pygame' )
def has_numpy( python ):
    return has_module( python, 'numpy' )
        
def ensure_virtualenv( python, numpy=True ):
    """Check/create virtualenv using our naming scheme"""
    expected_name = os.path.join( TEST_ENVS, python+['-nonum','-num'][bool(numpy)])
    log.info( 'Ensuring %s', expected_name )
    bin_path = os.path.join( expected_name, 'bin' )
    if not os.path.exists( expected_name ):
        if not os.path.exists( TEST_ENVS ):
            os.makedirs( TEST_ENVS )
        subprocess.check_call([
            'virtualenv', '-p', python, expected_name
        ])
    target_python = os.path.join( bin_path, 'python' )
    to_install = []
    if not has_pygame( target_python ):
        to_install.append( 'pygame' )
    if numpy and not has_numpy( target_python ):
        to_install.append( 'numpy' )
    if to_install:
        subprocess.check_call([
            os.path.join( bin_path, 'pip' ),
            'install',
            '-f', WHEEL_DIR,
            '--pre',
            '--no-index',
        ]+to_install)
    
    subprocess.check_call([
        os.path.join( bin_path, 'python' ),
        '-m', 'pip', 'install',
        '.',
    ], cwd = os.path.join( HERE, '..','OpenGL_accelerate' ))
    subprocess.check_call([
        os.path.join( bin_path, 'python' ),
        '-m', 'pip', 'install',
        '-e', '.',
    ], cwd = os.path.join( HERE, '..' ))

def env_setup():
    our_pythons = [p for p in PYTHONS if have_python(p)]
    for p in our_pythons:
        for numpy in [True,False]:
            ensure_virtualenv(p,numpy)

def main():
    env_setup()
    our_pythons = glob.glob( os.path.join( TEST_ENVS, '*','bin','python' ))
    our_pythons.sort()
    our_pythons.reverse()
    for flag in FLAGS:
        for python in our_pythons:
            for b in [True,False]:
                env = os.environ.copy()
                env['PYOPENGL_'+ flag] = str(b)
                command = [python,'test_core.py', '-v']
                if 'python2.6' not in python:
                    command.append( '-f')
                log.info( 'Starting PYOPENGL_%s=%s %s',flag, b, ' '.join(command))
                subprocess.check_call( command, env=env)

if __name__ == "__main__":
    logging.basicConfig( level = logging.INFO )
    main()



#! /usr/bin/env python3
import glfw
import OpenGL.GL as gl

glfw.init()
glfw.window_hint(glfw.CLIENT_API, glfw.OPENGL_ES_API)
glfw.window_hint(glfw.CONTEXT_VERSION_MAJOR, 3)
glfw.window_hint(glfw.CONTEXT_VERSION_MINOR, 1)
window = glfw.create_window(640, 480, 'Binder', None, None)
assert window
glfw.make_context_current(window)

# gl.glDispatchCompute(1,1,1) will trigger this for instance
print(gl.extensions.GLQuerier.pullVersion())



"""EGL Pygame test framework"""
from __future__ import print_function
import os

if not os.environ.get('PYOPENGL_PLATFORM'):
    os.environ['PYOPENGL_PLATFORM'] = 'egl'
import ctypes
import pygame.display
import pygame
import logging
import OpenGL
from functools import wraps

if os.environ.get('TEST_NO_ACCELERATE'):
    OpenGL.USE_ACCELERATE = False

from OpenGL import arrays
from OpenGL.EGL import *

log = logging.getLogger(__name__)

DESIRED_ATTRIBUTES = [
    EGL_BLUE_SIZE,
    8,
    EGL_RED_SIZE,
    8,
    EGL_GREEN_SIZE,
    8,
    EGL_DEPTH_SIZE,
    24,
    EGL_COLOR_BUFFER_TYPE,
    EGL_RGB_BUFFER,
    EGL_CONFIG_CAVEAT,
    EGL_NONE,  # Don't allow slow/non-conformant
]
API_BITS = {
    'opengl': EGL_OPENGL_BIT,
    'gl': EGL_OPENGL_BIT,
    'gles2': EGL_OPENGL_ES2_BIT,
    'gles1': EGL_OPENGL_ES_BIT,
    'gles': EGL_OPENGL_ES_BIT,
    'es2': EGL_OPENGL_ES2_BIT,
    'es1': EGL_OPENGL_ES_BIT,
    'es': EGL_OPENGL_ES_BIT,
}
API_NAMES = dict(
    [
        (
            k,
            {
                EGL_OPENGL_BIT: EGL_OPENGL_API,
                EGL_OPENGL_ES2_BIT: EGL_OPENGL_ES_API,
                EGL_OPENGL_ES_BIT: EGL_OPENGL_ES_API,
            }[v],
        )
        for k, v in API_BITS.items()
    ]
)


def egltest(size=(300, 300), name=None, api='es2', attributes=DESIRED_ATTRIBUTES):
    def gltest(function):
        """Decorator to allow a function to run in a Pygame GLES[1,2,3] context"""

        @wraps(function)
        def test_function(*args, **named):
            major, minor = ctypes.c_long(), ctypes.c_long()
            display = eglGetDisplay(EGL_DEFAULT_DISPLAY)
            eglInitialize(display, major, minor)
            num_configs = ctypes.c_long()
            configs = (EGLConfig * 2)()
            api_constant = API_NAMES[api.lower()]
            local_attributes = attributes[:]
            local_attributes.extend(
                [
                    EGL_CONFORMANT,
                    API_BITS[api.lower()],
                    EGL_NONE,
                ]
            )
            print('local_attributes', local_attributes)
            local_attributes = arrays.GLintArray.asArray(local_attributes)
            eglChooseConfig(display, local_attributes, configs, 2, num_configs)
            print('API', api_constant)
            eglBindAPI(api_constant)

            # now need to get a raw X window handle...
            pygame.init()
            pygame.display.set_mode(size)
            window = pygame.display.get_wm_info()['window']
            surface = eglCreateWindowSurface(display, configs[0], window, None)

            ctx = eglCreateContext(display, configs[0], EGL_NO_CONTEXT, None)
            if ctx == EGL_NO_CONTEXT:
                raise RuntimeError('Unable to create context')
            try:
                eglMakeCurrent(display, surface, surface, ctx)
                function(*args, **named)
                eglSwapBuffers(display, surface)
            finally:
                pygame.display.quit()
                pygame.quit()

        return test_function

    return gltest



from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

glutInit([''])
glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB)
print('OK')
try:
    if fgDeinitialize:
        fgDeinitialize(False)
except NameError as err:
    pass  # Older PyOpenGL, you may see a seg-fault here...



#! /usr/bin/env python
import logging

# import os_egl
from OpenGL.EGL import *
from OpenGL.EGL.EXT.device_enumeration import eglQueryDevicesEXT

log = logging.getLogger(__name__)


def main():
    devices = (EGLDeviceEXT * 10)()
    count = EGLint()
    major, minor = EGLint(), EGLint()
    if not eglQueryDevicesEXT:
        raise RuntimeError("No egl query devices extension available")
    if eglQueryDevicesEXT(10, devices, count):
        log.info("%s devices enumerated", count)
        for i, device in enumerate(devices[: count.value]):
            log.info("Describing device #%d: %s", i, device)
            display = eglGetDisplay(device)
            try:
                if not eglInitialize(display, major, minor):
                    log.info("Could not initialise, skipping")
                    continue
            except EGLError as err:
                log.warning("Unable to initialise EGL for display %d: %s", i, err)
            log.info("Display #%d, Version %s.%s", i, major.value, minor.value)
            for key in [
                EGL_VENDOR,
            ]:
                log.info(" Vendor: %s", eglQueryString(display, key))
            log.info("Finished device #%d", i)
    else:
        log.warning("Unable to query devices")


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    main()



#! /usr/bin/env python
from __future__ import print_function
import pygamegltest
from OpenGL.GL import glGetString, GL_SHADING_LANGUAGE_VERSION
from OpenGL._bytes import as_8_bit

@pygamegltest.pygametest(name='Get GL_SHADING_LANGUAGE_VERSION')
def show_glsl_version():
    version = glGetString( GL_SHADING_LANGUAGE_VERSION )
    version = version.split(as_8_bit(' '))[0]
    version = [int(x) for x in version.split(as_8_bit('.'))[:2]]
    return version 

if __name__ == "__main__":
    print( show_glsl_version() )




#! /usr/bin/env python3
import os, logging, glob, ctypes
os.environ['PYOPENGL_PLATFORM'] = 'egl'
log = logging.getLogger(__name__)
import OpenGL
from OpenGL.EGL import gbmdevice
from OpenGL.EGL import *
from OpenGL.EGL.EXT.platform_base import *
from OpenGL.EGL.MESA.platform_gbm import *

def main():
    for device in gbmdevice.enumerate_devices():
        log.info("Checking device: %s", device)
        device = gbmdevice.open_device(device)
        try:
            display = eglGetPlatformDisplayEXT(
                EGL_PLATFORM_GBM_MESA, 
                device, 
                ctypes.c_void_p(0)
            )
            if display == EGL_NO_DISPLAY:
                log.error("Failed to get platform display for %s", display)
            else:
                log.info("Got platform display %s", display)
        finally:
            gbmdevice.close_device(device)

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    main()



"""Test cases that run stand-alone check-scripts"""

import os, sys, subprocess, logging
import pytest
from functools import wraps
from OpenGL.GLUT import glutInit

WAYLAND = os.environ.get('XDG_SESSION_TYPE') == 'wayland'

try:
    import numpy
except ImportError:
    numpy = None
HERE = os.path.dirname(os.path.abspath(__file__))
log = logging.getLogger(__name__)


def glx_only(func):
    @wraps(func)
    def glx_only_test(*args, **named):
        if WAYLAND:
            pytest.skip('GLX is not wayland compatible generally')
        if not sys.platform in ('linux', 'linux2'):
            pytest.skip('Linux-only')
        return func(*args, **named)

    return glx_only_test

def xlib_only(func):
    @wraps(func)
    def xlib_only_test(*args, **named):
        if WAYLAND:
            pytest.skip('Raw XLIB operations do not work on wayland')
        return func(*args, **named)

    return xlib_only_test


def glut_only(func):
    @wraps(func)
    def glut_only_test(*args, **named):
        if WAYLAND:
            pytest.skip('GLUT has poor wayland support')
        if not glutInit:
            pytest.skip('No GLUT installed')
        return func(*args, **named)

    return glut_only_test


def numpy_only(func):
    @wraps(func)
    def glut_only_test(*args, **named):
        if not numpy:
            pytest.skip('No GLUT installed')
        return func(*args, **named)

    return glut_only_test


def check_test(func):
    filename = func.__name__[5:] + '.py'
    file = os.path.join(HERE, filename)

    @wraps(func)
    def test_x():
        log.info('Starting test: %s', filename)
        pipe = subprocess.Popen(
            [
                sys.executable,
                file,
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        try:
            stdout, stderr = pipe.communicate()
        except subprocess.TimeoutExpired:
            log.warning('TIMEOUT on %s', filename)
            pipe.kill()
            raise
        except subprocess.CalledProcessError as err:
            log.warning('ERROR reported by process: %s', err)
            raise
        output = stdout.decode('utf-8', errors='ignore')
        lines = [x.strip() for x in output.strip().splitlines()]
        if not lines:
            log.error(
                'Test did not produce output: %s',
                stderr.decode('utf-8', errors='ignore'),
            )
            raise RuntimeError('Test script failure on %s' % (func.__name__))
        if 'SKIP' in lines:
            raise pytest.skip('Skipped by executable on %s' % (func.__name__))
        elif 'OK' in lines:
            return
        else:
            log.error(
                'Failing check script stderr: %s',
                stderr.decode('utf-8', errors='ignore'),
            )
            log.error(
                'Failing check script stdout: %s',
                output,
            )
            raise RuntimeError('Test Failed')

    return test_x


@glut_only
@check_test
def test_check_crash_on_glutinit():
    """Checks that basic glut init works"""


@numpy_only
@xlib_only
@check_test
def test_check_egl_es1():
    """Checks egl with es1 under pygame"""


@numpy_only
@xlib_only
@check_test
def test_check_egl_es2():
    """Checks egl with es2 under pygame"""


@numpy_only
@xlib_only
@check_test
def test_check_egl_opengl():
    """Checks egl with opengl under pygame"""


@xlib_only
@check_test
def test_check_egl_platform_ext():
    """Checks egl display platform directly from render devices"""


@glut_only
@check_test
def test_check_glutwindow():
    """Checks GLUT window manipulation functions"""


@pytest.mark.xfail
@check_test
def test_check_egl_pygame():
    """Checks egl running over a pygame context"""


@glut_only
@check_test
def test_check_freeglut_deinit():
    """Checks free-glut deinitialise"""


@check_test
def test_check_import_err():
    """Checks that the GLU module can be imported"""


@numpy_only
@check_test
def test_check_leak_on_discontiguous_array():
    """Checks that discontiguous array copy doesn't leak the copy"""


@check_test
def test_check_init_framebufferarb():
    """Checks that framebufferarb init function is non-null"""


@check_test
def test_check_gles_imports():
    """Checks that we can import GLES without crashing"""


@glut_only
@check_test
def test_check_glut_debug():
    """Tests GLUT debug function"""


@glut_only
@check_test
def test_check_glut_fc():
    """Tests GLUT forward-compatible-only"""


@glut_only
@check_test
def test_check_glut_load():
    """Tests GLUT forward-compatible-only"""


@glut_only
@check_test
def test_check_glutinit():
    """Tests GLUT init"""


@glut_only
@check_test
def test_check_glutinit_0args():
    """Tests GLUT init with no arguments"""


@glut_only
@check_test
def test_check_glutinit_single():
    """Tests GLUT init with single argument"""


@glut_only
@check_test
def test_check_glutinit_simplest():
    """Tests GLUT init in simplest possible case"""


@check_test
def test_check_silence_numpy_warning():
    """Tests GLUT init in simplest possible case"""


@check_test
def test_egl_ext_enumerate():
    """Tests that EGL can retrieve extension list"""


@check_test
def test_feedbackvarying():
    """Tests that feedback varying buffer operations work"""


@check_test
def test_test_sf2946226():
    """Test sourceforge bug vs. regressions"""


@glut_only
@check_test
def test_test_instanced_draw_detect():
    """Test that instanced draw extension can be identified"""


@glut_only
@check_test
def test_test_gldouble_ctypes():
    """Test use of gldouble array in ctypes"""


@check_test
def test_test_glgetactiveuniform():
    """Test use of gldouble array in ctypes"""


@pytest.mark.xfail(
    condition=sys.version_info[:2] > (3, 11),
    reason='Python 3.12+ JIT consumes RAM during this test which the crude memory bookkeeping cannot track',
)
@check_test
def test_test_glgetfloat_leak():
    """Test use of gldouble array in ctypes"""



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_win32_keyed_mutex'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_win32_keyed_mutex',error_checker=_errors._error_checker)

@_f
@_p.types(_cs.GLboolean,_cs.GLuint,_cs.GLuint64,_cs.GLuint)
def glAcquireKeyedMutexWin32EXT(memory,key,timeout):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint,_cs.GLuint64)
def glReleaseKeyedMutexWin32EXT(memory,key):pass



#! /usr/bin/env python
from __future__ import print_function
import xlibegltest as egltest
from numpy import array
from OpenGL.arrays.vbo import VBO
from OpenGL.arrays import ArrayDatatype
from OpenGL.GLES2 import *
from OpenGL.GLES2 import shaders


@egltest.egltest(api='es2')
def test_gl():
    glClearColor(1, 0, 0, 0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

    print('Error before compilation?', glGetError())
    shader = shaders.compileProgram(
        shaders.compileShader(
            '''#version 130
    attribute vec3 position;
    void main() {
        gl_Position = vec4( position, 0 );
    }''',
            GL_VERTEX_SHADER,
        ),
        shaders.compileShader(
            '''#version 130
    void main() {
        gl_FragColor = vec4( 0,1,0,.5 );
    }''',
            GL_FRAGMENT_SHADER,
        ),
    )
    vbo = VBO(
        array(
            [
                (0, 1, 0),
                (1, -1, 0),
                (-1, -1, 0),
            ],
            dtype='f',
        )
    )
    position_location = glGetAttribLocation(shader, 'position')
    stride = 3 * 4
    with vbo:
        with shader:
            glEnableVertexAttribArray(position_location)
            stride = 3 * 4
            glVertexAttribPointer(position_location, 3, GL_FLOAT, False, stride, vbo)
            glDrawArrays(GL_TRIANGLES, 0, 3)
    print('OK')


if __name__ == "__main__":
    test_gl()



from OpenGL.GLUT import *
from OpenGL.GL import *


def main():
    if glutInit:
        glutInit(' ')
        glutInitDisplayMode(GLUT_SINGLE)
        window = glutCreateWindow("hello")
        glutDisplayFunc(lambda *args: 1)
        # glutMainLoop()
        try:
            if fgDeinitialize:
                fgDeinitialize(False)
        except NameError as err:
            pass  # Older PyOpenGL, you may see a seg-fault here...
        print('OK')
    else:
        print('SKIP')


if __name__ == "__main__":
    main()



"""Helper to parse limited subset of C for wrapper operations"""
import re,logging,keyword
from ctypetopytype import ctype_to_pytype
log = logging.getLogger( __name__ )

reserved_names = set(keyword.kwlist)

class Helper( object ):
    def __getitem__( self, key ):
        item = getattr( self, key, None )
        if item is None:
            raise KeyError( key )
        if callable( item ):
            return item()
        else:
            return item

class Function( Helper ):
    """Parse function parameters from C-style declaration"""
    def __init__( self, returnType, name, signature):
        """Parse definition into our various elements"""
        self.name = name
        self.returnType = self.parseReturnType(returnType)
        try:
            self.argTypes, self.argNames = self.parseArguments( signature )
        except Exception, err:
            log.error( """Error parsing arguments for %s %s: %s""", name, signature, err )
            self.argTypes, self.argNames = (), ()
    findName = re.compile( '[a-zA-z0-9]+$' )
    def parseReturnType( self, returnType ):
        return self.cTypeToPyType( returnType )
    def parseArguments( self, signature ):
        """Parse a C argument-type declaration into a ctypes-style argTypes and argNames"""
        signature = signature.strip()
        if signature.startswith('(') and signature.endswith(')'):
            signature = signature[1:-1]
        # first and easiest case is a void call...
        if not signature.strip() or signature.strip() == 'void':
            return (), ()
        types, names = [], []
        for i,item in enumerate(signature.split( ',' )):
            # TODO: have to hack around the official header having junk here...
            if item.strip() == 'EGLSyncKHR':
                item = 'EGLSyncKHR sync'
            item = item.strip()
            nameMatch = self.findName.search( item )
            if not nameMatch:
                name = 'arg_%i'%i
                rest = item
            else:
                name = nameMatch.group(0)
                rest = item[:nameMatch.start(0)].strip()
                if not rest:
                    rest = name 
                    name = 'arg_%i'%i
            if name in reserved_names:
                name = name + '_'
            types.append( self.cTypeToPyType( rest ) )
            names.append( name )
        return types, names
    cTypeToPyType = staticmethod( ctype_to_pytype )
    
    def errorReturn( self ):
        return '0'
    def declaration( self ):
        """Produce a declaration for this function in ctypes format"""
        returnType = self.returnType
        if self.argTypes:
            argTypes = ','.join(self.argTypes)
        else:
            argTypes = ''
        if self.argNames:
            argNames = ','.join(self.argNames)
        else:
            argNames = ''
        arguments = ', '.join([
            '%(type)s(%(name)s)'%locals()
            for (type,name) in [
                (type.split('.',1)[1],name)
                for type,name in zip( self.argTypes,self.argNames )
            ]
        ])
        name = self.name 
        if returnType.strip() in ('_cs.GLvoid', '_cs.void'):
            returnType = pyReturn = 'None'
        else:
            pyReturn = self.returnType
        log.info( 'returnType %s -> %s', self.returnType, pyReturn )
        doc = '%(name)s(%(arguments)s) -> %(pyReturn)s'%locals()
        return self.TEMPLATE%locals()
    TEMPLATE = """@_f
@_p.types(%(returnType)s,%(argTypes)s)
def %(name)s(%(argNames)s):pass"""



from __future__ import print_function
import unittest, pygame, pygame.display, time, os
import logging 
logging.basicConfig(level=logging.INFO)
HERE = os.path.dirname( __file__ )
import pickle
try:
    import cPickle
except ImportError as err:
    cPickle = pickle

try:
    from numpy import *
except ImportError as err:
    array = None

pygame.display.init()
import OpenGL 
if os.environ.get( 'TEST_NO_ACCELERATE' ):
    OpenGL.USE_ACCELERATE = False
#OpenGL.FULL_LOGGING = True
OpenGL.CONTEXT_CHECKING = True
OpenGL.FORWARD_COMPATIBLE_ONLY = False
OpenGL.UNSIGNED_BYTE_IMAGES_AS_STRING = True

#from OpenGL._bytes import bytes, _NULL_8_BYTE, unicode, as_8_bit
from OpenGL.GL import *
try:
    glGetError()
except error.NoContext as err:
    # good, should have got this error 
    pass
else:
    print( 'WARNING: Failed to catch invalid context' )
    #raise RuntimeError( """Did not catch invalid context!""" )
#from OpenGL import error
from OpenGL.GLU import *
#from OpenGL.arrays import arraydatatype
import OpenGL
from OpenGL.extensions import alternate
#import ctypes
from OpenGL.GL.framebufferobjects import *
from OpenGL.GL.EXT.multi_draw_arrays import *
from OpenGL.GL.ARB.imaging import *

glMultiDrawElements = alternate( 
    glMultiDrawElementsEXT, glMultiDrawElements, 
)

class BaseTest( unittest.TestCase ):
    width = height = 300
    def setUp( self ):
        """Set up the operation"""
        
        self.screen = pygame.display.set_mode(
            (self.width,self.height),
            pygame.OPENGL | pygame.DOUBLEBUF,
        )
        
        pygame.display.set_caption('Testing system')
        pygame.key.set_repeat(500,30)
        glMatrixMode (GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(40.0, 300/300., 1.0, 20.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        gluLookAt(
            -2,0,3, # eyepoint
            0,0,0, # center-of-view
            0,1,0, # up-vector
        )
        glClearColor( 0,0,.25, 0 )
        glClear( GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT )
    
    def tearDown( self ):
        glFlush()
        pygame.display.flip()
        # this is just so that you can see the effect
        # before we run the next test...
        time.sleep( .05 )



#! /usr/bin/env python
"""PyOpenGL setup script distutils/setuptools/pip based"""
import sys, os
from setuptools import setup

if sys.platform == "win32":
    # binary versions of GLUT and GLE for Win32 (sigh)
    DLL_DIRECTORY = os.path.join("OpenGL", "DLLS")
    datafiles = [
        (
            DLL_DIRECTORY,
            [
                os.path.join(DLL_DIRECTORY, file)
                for file in os.listdir(DLL_DIRECTORY)
                if os.path.isfile(os.path.join(DLL_DIRECTORY, file))
            ],
        ),
    ]
else:
    datafiles = []


if __name__ == "__main__":
    setup(
        options={
            "sdist": {
                "formats": ["gztar"],
                "force_manifest": True,
            },
        },
        data_files=datafiles,
        include_package_data=True,
    )



#! /usr/bin/env python
import logging, os, subprocess
log = logging.getLogger( 'checker' )
def main():
    env = running_env = os.environ.copy()
    for path,dirs,files in os.walk( os.path.join('..','OpenGL','raw') ):
        for file in files:
            if file.endswith( '.py' ):
                log.debug( 'Check: %s', file )
                if 'osmesa' in path.split(os.sep):
                    running_env = env.copy()
                    running_env['PYOPENGL_PLATFORM'] = 'osmesa'
                elif 'EGL' in path.split(os.sep):
                    running_env = env.copy()
                    running_env['PYOPENGL_PLATFORM'] = 'egl'
                elif 'WGL' in path.split(os.sep):
                    log.debug("Skipping win32")
                    continue
                    
                else:
                    running_env = env
                try:
                    subprocess.check_call( ['python', os.path.join( path, file )],env=running_env )
                except subprocess.CalledProcessError:
                    log.error( 'Failure loading: %s/%s', path, file )

if __name__ == "__main__":
    logging.basicConfig( level = logging.INFO )
    main()



import basetestcase
import os
from OpenGL.GL import *

try:
    import numpy as np
except ImportError as err:
    np = None
from OpenGL.arrays import arraydatatype

HERE = os.path.abspath(os.path.dirname(__file__))
from OpenGL.GL.ARB import texture_rg


class TestTextures(basetestcase.BaseTest):
    def test_enable_histogram(self):
        if glInitImagingARB():
            glHistogram(GL_HISTOGRAM, 256, GL_LUMINANCE, GL_FALSE)
            glEnable(GL_HISTOGRAM)
            glDisable(GL_HISTOGRAM)
        else:
            print('No ARB imaging extension')

    if np:

        def test_glreadpixels_warray(self):
            """SF#1311265 allow passing in the array object"""
            width, height = self.width, self.height
            data = np.zeros((width, height, 3), 'B')
            image1 = glReadPixelsub(0, 0, width, height, GL_RGB, array=data)
            assert image1 is not None

        someData = [(0, 255, 0)]

        def test_glAreTexturesResident(self):
            """Test that PyOpenGL api for glAreTexturesResident is working

            Note: not currently working on AMD64 Linux for some reason
            """
            textures = glGenTextures(2)
            residents = []
            data = np.array(self.someData, 'i')
            for texture in textures:
                glBindTexture(GL_TEXTURE_2D, int(texture))
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, 1, 1, 0, GL_RGB, GL_INT, data)
                residents.append(
                    glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_RESIDENT)
                )
            glGetError()
            result = glAreTexturesResident(textures)
            assert len(result) == 2
            for tex, expected, found in zip(textures, residents, result):
                if expected != found:
                    print(
                        (
                            'Warning: texture %s residence expected %s got %s'
                            % (tex, expected, found)
                        )
                    )

    def test_glreadpixelsf(self):
        """Issue #1979002 crash due to mis-calculation of resulting array size"""
        width, height = self.width, self.height
        readback_image1 = glReadPixelsub(0, 0, width, height, GL_RGB)
        assert readback_image1 is not None
        readback_image2 = glReadPixelsf(0, 0, width, height, GL_RGB)
        assert readback_image2 is not None

    def test_glreadpixels_is_string(self):
        """Issue #1959860 incompatable change to returning arrays reversed"""
        width, height = self.width, self.height
        readback_image1 = glReadPixels(0, 0, width, height, GL_RGB, GL_UNSIGNED_BYTE)
        assert isinstance(readback_image1, bytes), type(readback_image1)
        readback_image1 = glReadPixels(0, 0, width, height, GL_RGB, GL_BYTE)
        assert not isinstance(readback_image1, bytes), type(readback_image2)

    def test_passBackResults(self):
        """Test ALLOW_NUMPY_SCALARS to allow numpy scalars to be passed in"""
        textures = glGenTextures(2)
        glBindTexture(GL_TEXTURE_2D, textures[0])

    def test_nullTexture(self):
        """Test that we can create null textures"""
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, 512, 512, 0, GL_RGB, GL_INT, None)

    def test_get_boolean_bitmap(self):
        # should not raise error
        glGetBoolean(GL_TEXTURE_2D)

    if np:

        def test_draw_bitmap_pixels(self):
            """SF#2152623 Drawing pixels as bitmaps (bits)"""
            # this core-dumps on Mesa Intel on Ubuntu 15.04 :(
            # nosetest skip would be more appropriate
            pixels = np.array([0, 0, 0, 0, 0, 0, 0, 0], 'B')
            glDrawPixels(8, 8, GL_COLOR_INDEX, GL_BITMAP, pixels)

    def test_get_max_tex_units(self):
        """SF#2895081 glGetIntegerv( GL_MAX_TEXTURE_IMAGE_UNITS )"""
        units = glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS)
        assert units

    def test_glGenTextures(self):
        texture = glGenTextures(1)
        assert texture

    def test_createTargetArray(self):
        import OpenGL.GL as gl
        import OpenGL.images as images

        size = (640, 480)
        array1 = images.createTargetArray(
            gl.GL_BGRA, size, gl.GL_UNSIGNED_INT_8_8_8_8_REV
        )
        array2 = images.createTargetArray(gl.GL_RGBA, size, gl.GL_UNSIGNED_BYTE)
        array3 = images.createTargetArray(
            gl.GL_RGBA, size, gl.GL_UNSIGNED_INT_8_8_8_8_REV
        )
        if hasattr(array1, 'nbytes'):
            assert array1.nbytes == array3.nbytes
            assert array1.nbytes == array2.nbytes
        else:
            assert ctypes.sizeof(array1) == ctypes.sizeof(array3)
            assert ctypes.sizeof(array1) == ctypes.sizeof(array2)

        try:
            images.createTargetArray(gl.GL_RGBA, size, gl.GL_UNSIGNED_BYTE_3_3_2)
        except ValueError as err:
            pass
        else:
            raise RuntimeError(
                """Should have failed with insufficient components in the type to hold the format"""
            )

    def test_rg_format(self):
        # Note: this is actually only known after context creation...
        if not texture_rg.glInitTextureRgARB():
            return
        texture = glGenTextures(1)
        data = arraydatatype.GLfloatArray.asArray([0.3, 0.5])
        glBindTexture(GL_TEXTURE_2D, int(texture))
        glTexImage2D(GL_TEXTURE_2D, 0, texture_rg.GL_RG, 1, 1, 0, GL_RG, GL_FLOAT, data)



"""Test for github issue #47"""

import sys
import OpenGL

OpenGL.SIZE_1_ARRAY_UNPACK = False  # just for convenience
OpenGL.ERROR_ON_COPY = False  # we are checking a leak in the copying
import pygamegltest
import numpy as np
from OpenGL.GL import *
from OpenGL.GL import shaders
from sys import getrefcount


@pygamegltest.pygametest(name="Texture image 2d leak check")
def main():
    data = np.zeros([256, 256, 3], dtype='b')
    glEnable(GL_TEXTURE_2D)
    textures = glGenTextures(1)
    glBindTexture(GL_TEXTURE_2D, textures[0])
    reversed_data = data[::-1]
    assert not reversed_data.flags['C_CONTIGUOUS']
    rc1 = getrefcount(reversed_data)
    for i in range(100):
        glTexImage2D(
            GL_TEXTURE_2D,
            0,
            GL_RGB,
            256,
            256,
            0,
            GL_RGB,
            GL_UNSIGNED_BYTE,
            reversed_data,
        )
    rc2 = getrefcount(reversed_data)
    assert rc1 == rc2, (rc1, rc2)
    sys.stdout.write('OK\n')
    sys.stdout.flush()


if __name__ == "__main__":
    main()



from __future__ import print_function
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GL.shaders import *
from OpenGL.GL.ARB.shader_objects import glGetActiveUniformARB

vertex_shader = """
uniform float scale;
void main(void)
{
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex * scale;
}
"""


def main():
    pygame.init()
    disp = pygame.display.set_mode((1024, 768), OPENGL | DOUBLEBUF)

    program = compileProgram(compileShader(vertex_shader, GL_VERTEX_SHADER))

    nu = glGetProgramiv(program, GL_ACTIVE_UNIFORMS)
    for i in range(nu):
        name, size, type = glGetActiveUniform(program, i)
        print('CORE - ', name, size, type)
        glGetActiveUniformARB(program, i)
        print('ARB  - ', name, size, type)
    print('OK')


if __name__ == "__main__":
    main()
